@@ -42,6 +42,11 @@ MainWindow::MainWindow(QWidget *parent)
     connect(gpsAct, SIGNAL(triggered()), this, SLOT(gpsButtonClicked()));
     menuBar()->addAction(gpsAct);
 
+    //Toggle GPS Button
+    QAction *donateAct = new QAction(tr("Donate"), this);
+    connect(donateAct, SIGNAL(triggered()), this, SLOT(donateButtonClicked()));
+    menuBar()->addAction(donateAct);
+
     setWindowTitle("Fahrplan");
 
     //Main Window Layout
@@ -84,11 +89,11 @@ MainWindow::MainWindow(QWidget *parent)
     QHBoxLayout *datetimeHLayout = new QHBoxLayout;
     QWidget     *datetimeHContainer = new QWidget;
 
-    dateSelector  = new QMaemo5ValueButton("Date");
+    dateSelector  = new QMaemo5ValueButton(tr("Date"));
     dateSelector->setPickSelector(new QMaemo5DatePickSelector());
     dateSelector->setValueLayout(QMaemo5ValueButton::ValueUnderTextCentered);
 
-    timeSelector  = new QMaemo5ValueButton("Time");
+    timeSelector  = new QMaemo5ValueButton(tr("Time"));
     timeSelector->setPickSelector(new QMaemo5TimePickSelector());
     timeSelector->setValueLayout(QMaemo5ValueButton::ValueUnderTextCentered);
 
@@ -98,6 +103,9 @@ MainWindow::MainWindow(QWidget *parent)
     datetimeHLayout->addWidget(timeSelector);
     datetimeHContainer->setLayout(datetimeHLayout);
 
+    QHBoxLayout *optionsHLayout = new QHBoxLayout;
+    QWidget     *optionsHContainer = new QWidget;
+
     //Departure/Arrival Selector Button
     QStandardItemModel * departureModel = new QStandardItemModel(0, 1);
     QStandardItem *item = new QStandardItem(tr("Arrival time"));
@@ -115,6 +123,16 @@ MainWindow::MainWindow(QWidget *parent)
     modeSelector->setCurrentIndex(1);
     modeButton->setPickSelector(modeSelector);
 
+    //Restrict Train Types
+    traintypesButton = new QMaemo5ValueButton(tr("Restrict train types"));
+    traintypesButton->setValueLayout(QMaemo5ValueButton::ValueUnderTextCentered);
+
+    optionsHLayout->setMargin(0);
+    optionsHLayout->setSpacing(0);
+    optionsHLayout->addWidget(modeButton);
+    optionsHLayout->addWidget(traintypesButton);
+    optionsHContainer->setLayout(optionsHLayout);
+
     //Search Button
     searchButton = new QPushButton();
     searchButton->setText(tr("Search"));
@@ -127,7 +145,7 @@ MainWindow::MainWindow(QWidget *parent)
     layout->setStretch(0, 2);
     layout->addWidget(stationsHContainer);
     layout->addWidget(datetimeHContainer);
-    layout->addWidget(modeButton);
+    layout->addWidget(optionsHContainer);
     layout->addWidget(searchButton);
     mainWidget->setLayout(layout);
 
@@ -140,6 +158,8 @@ MainWindow::MainWindow(QWidget *parent)
 
     //Init GPS if in settings
     locationDataSource = QGeoPositionInfoSource::createDefaultSource(this);
+    locationDataSource->setPreferredPositioningMethods(locationDataSource->AllPositioningMethods);
+
     QObject::connect(locationDataSource, SIGNAL(positionUpdated(QGeoPositionInfo)), this, SLOT(positionUpdated(QGeoPositionInfo)));
 
     if (settings->value("autostartGPS", true).toBool() ==  true)
@@ -151,6 +171,23 @@ MainWindow::MainWindow(QWidget *parent)
         locationDataSourceIsUpdating = false;
     }
 
+    //Prefill stations
+    if ((settings->value("prefillsearchbox", true).toBool() ==  true) &&
+        (settings->value("storelastsearch", true).toBool() ==  true)
+        )
+    {
+        QStringList stationsList;
+        stationsList = settings->value("lastsearch/stations", stationsList).toStringList();
+        //we reverse the list to get the latest stations on the top
+        stationsList = fahrplanUtils::reverseQStringList(stationsList);
+
+        if (stationsList.count() > 1)
+        {
+            stationDeparture->setValueText(stationsList[1]);
+            stationArrival->setValueText(stationsList[0]);
+        }
+    }
+
     initBackend();
 
     //Show first launch msg
@@ -158,7 +195,7 @@ MainWindow::MainWindow(QWidget *parent)
     {
         QMessageBox::about(this,
                            tr("Welcome to a new version of fahrplan this has changed"),
-                           tr("- gps fixes for bahn.de backend<br>- if you in details screen youre able to add the infos to n900 default calendar<br>- optimized usage of OSM gps interface.")
+                           tr("- bahn.de: adding train restrictions<br>- Calendar: you can select to which calendar you want add the journey.<br>- last used stationnames could be filled automatically on startup.<br>- Added a Donate link to the mainmenu")
                            );
 
         settings->setValue("firstLaunch", APP_VER);
@@ -188,13 +225,35 @@ void MainWindow::initBackend()
        backendParser = new parserMobileBahnDe();
        setWindowTitle("Fahrplan (bahn.de)");
     }
+
+    //Update gui based on Backend
+    QStringList trainRestrictions = backendParser->getTrainRestrictions();
+    traintypesButton->setPickSelector(NULL);
+    traintypesButton->setValueText(tr("not supported"));
+    traintypesButton->setEnabled(false);
+    if (trainRestrictions.count() > 0)
+    {
+        QStandardItemModel * trainrestrModel = new QStandardItemModel(0, 1);
+        for (int i = 0; i < trainRestrictions.count(); i++)
+        {
+            QStandardItem *item = new QStandardItem(trainRestrictions[i]);
+            item->setTextAlignment(Qt::AlignCenter);
+            item->setEditable(false);
+            trainrestrModel->appendRow(item);
+        }
+        QMaemo5ListPickSelector *trainrestrSelector = new QMaemo5ListPickSelector;
+        trainrestrSelector->setModel(trainrestrModel);
+        trainrestrSelector->setCurrentIndex(0);
+        traintypesButton->setPickSelector(trainrestrSelector);
+        traintypesButton->setEnabled(true);
+    }
 }
 
 void MainWindow::positionUpdated(QGeoPositionInfo geoPositionInfo)
 {
     if (!myPositionInfo.isValid() && geoPositionInfo.isValid())
     {
-        QMaemo5InformationBox::information(this, "<br>GPS is now available!<br>", QMaemo5InformationBox::DefaultTimeout);
+        QMaemo5InformationBox::information(this, tr("<br>GPS is now available!<br>"), QMaemo5InformationBox::DefaultTimeout);
     }
     myPositionInfo = geoPositionInfo;
 }
@@ -205,12 +264,12 @@ void MainWindow::gpsButtonClicked()
     {
         locationDataSource->stopUpdates();
         locationDataSourceIsUpdating = false;
-        QMaemo5InformationBox::information(this, "<br>GPS is now <b>disabled</b><br>", QMaemo5InformationBox::DefaultTimeout);
+        QMaemo5InformationBox::information(this, tr("<br>GPS is now <b>disabled</b><br>"), QMaemo5InformationBox::DefaultTimeout);
     } else
     {
         locationDataSource->startUpdates();
         locationDataSourceIsUpdating = true;
-        QMaemo5InformationBox::information(this, "<br>GPS is now <b>enabled</b><br>", QMaemo5InformationBox::DefaultTimeout);
+        QMaemo5InformationBox::information(this, tr("<br>GPS is now <b>enabled</b><br>"), QMaemo5InformationBox::DefaultTimeout);
     }
 }
 
@@ -233,11 +292,17 @@ void MainWindow::stationArrivalClicked()
     selectDialog->show();
 }
 
-void MainWindow::alarmButtonClicked()
+void MainWindow::addToCalendar(QString calendarname)
 {
     CMulticalendar *multiCalendar = CMulticalendar::MCInstance();
 
-    CCalendar *calendar = multiCalendar->getDefaultCalendar();
+    int pErrorCode;
+    CCalendar *calendar = multiCalendar->getCalendarByName(calendarname.toStdString(), pErrorCode);
+
+    if (!calendar)
+    {
+        return;
+    }
 
     QString title       = tr("Journey: ") + lastSearchDetails.items.first().fromStation + tr(" to ") + lastSearchDetails.items.last().toStation;
     QString description = lastSearchDetails.items.first().fromStation + tr(" to ") + lastSearchDetails.items.last().toStation + "\n";
@@ -285,7 +350,7 @@ void MainWindow::alarmButtonClicked()
         description.append("\n\n");
     }
 
-    description.append("\n\n(added by fahrplan app, please recheck informations before travel.)");
+    description.append(tr("\n\n(added by fahrplan app, please recheck informations before travel.)"));
 
     title       = fahrplanUtils::removeUmlauts(title);
     description = fahrplanUtils::removeUmlauts(description);
@@ -295,7 +360,7 @@ void MainWindow::alarmButtonClicked()
 
     if (tsFrom.toTime_t() < QDateTime::currentDateTime().toTime_t())
     {
-        QMaemo5InformationBox::information(this, "<br>Entry is in the past!<br>", QMaemo5InformationBox::DefaultTimeout);
+        QMaemo5InformationBox::information(this, tr("<br>Entry is in the past!<br>"), QMaemo5InformationBox::DefaultTimeout);
         return;
     } else
     {
@@ -309,14 +374,73 @@ void MainWindow::alarmButtonClicked()
 
         multiCalendar->addEvent(pEvent, calendar->getCalendarId(), errorCode);
 
-        QMaemo5InformationBox::information(this, "<br>Entry added to your default calendar.<br>", QMaemo5InformationBox::DefaultTimeout);
+        QMaemo5InformationBox::information(this, tr("<br>Entry added to the ") + calendarname + tr(" calendar.<br>"), QMaemo5InformationBox::DefaultTimeout);
     }
 }
 
+void MainWindow::selectTargetCalendar(QStringList items)
+{
+    //If only one entry found we directly use it.
+    if (items.count() == 1)
+    {
+        addToCalendar(items[0]);
+        this->hide();
+        return;
+    }
+
+    QStandardItemModel * tmpModel = new QStandardItemModel(0, 1);
+
+    QString string;
+    foreach(string, items)
+    {
+        //qDebug() << "- " << string;
+        QStandardItem *item = new QStandardItem(string);
+        item->setTextAlignment(Qt::AlignCenter);
+        item->setEditable(false);
+        tmpModel->appendRow(item);
+    }
+
+    QMaemo5ValueButton *tmpButton = new QMaemo5ValueButton(tr("Select target calendar"));
+    tmpButton->setValueLayout(QMaemo5ValueButton::ValueBesideText);
+    QMaemo5ListPickSelector *tmpSelector = new QMaemo5ListPickSelector;
+    connect(tmpSelector, SIGNAL(selected(QString)), this, SLOT(addToCalendar(QString)));
+    tmpSelector->setModel(tmpModel);
+    tmpButton->setPickSelector(tmpSelector);
+    tmpButton->click();
+}
+
+void MainWindow::alarmButtonClicked()
+{
+    CMulticalendar *multiCalendar = CMulticalendar::MCInstance();
+    vector<CCalendar*> vector_of_cal  = multiCalendar->getListCalFromMc();
+    QVector<CCalendar*> qvector_of_cal = QVector< CCalendar  *>::fromStdVector(vector_of_cal); //
+    if(!qvector_of_cal.empty())
+    {
+        QStringList calendars;
+        foreach(CCalendar *ccal, qvector_of_cal )
+        {
+            if (!ccal->IsReadOnly())
+            {
+                calendars.append(QString::fromStdString(ccal->getCalendarName()));
+            }
+        }
+
+        selectTargetCalendar(calendars);
+    }
+}
+
+void MainWindow::donateButtonClicked()
+{
+    QDesktopServices::openUrl(QUrl("https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&hosted_button_id=DW3LGJ6VEKFVS"));
+    QMessageBox::about(this,
+                       tr("Donate"),
+                       tr("Thank you for Donating!"));
+}
+
 void MainWindow::aboutButtonClicked()
 {
     QString version = APP_VER;
-    QString aboutMsg = "by smurfy (maemo@smurfy.de)<br>Version " + version + "<br><br>Current supported databackends:<br><b>bahn.de</b>, <b>mvv-muenchen.de</b>, <b>sbb.ch</b><br><br>This or prior versions uses code contribution by:<br><i>hcm</i>, <i>thp</i>, <i>qwerty12</i>, <i>qbast</i>";
+    QString aboutMsg = tr("by smurfy (maemo@smurfy.de)<br>Version ") + version + tr("<br><br>Current supported databackends:<br><b>bahn.de</b>, <b>mvv-muenchen.de</b>, <b>sbb.ch</b><br><br>This or prior versions uses code contribution by:<br><i>hcm</i>, <i>thp</i>, <i>qwerty12</i>, <i>qbast</i>");
     QMessageBox::about(this,
                        tr("About Fahrplan"),
                        tr(aboutMsg.toAscii()));
@@ -345,7 +469,7 @@ void MainWindow::resultItemClicked()
         showDetailsWindow();
     } else
     {
-        QMaemo5InformationBox::information(this, "<br>Details allready loading, please wait.<br>", QMaemo5InformationBox::DefaultTimeout);
+        QMaemo5InformationBox::information(this, tr("<br>Details already loading, please wait.<br>"), QMaemo5InformationBox::DefaultTimeout);
         return;
     }
 }
@@ -359,13 +483,13 @@ void MainWindow::showDetailsWindow()
 
     if (lastSearchDetails.items.count() == 0)
     {
-        QMaemo5InformationBox::information(this, "<br>Error opening the details.<br>" + lastSearchResult.errorMsg, QMaemo5InformationBox::DefaultTimeout);
+        QMaemo5InformationBox::information(this, tr("<br>Error opening the details.<br>") + lastSearchResult.errorMsg, QMaemo5InformationBox::DefaultTimeout);
         return;
     }
 
     //Details Window
     detailsWindow = new QMainWindow(resultWindow);
-    detailsWindow->setWindowTitle("Details");
+    detailsWindow->setWindowTitle(tr("Details"));
     #ifdef Q_WS_MAEMO_5
         detailsWindow->setAttribute(Qt::WA_Maemo5StackedWindow);
     #endif
@@ -456,7 +580,7 @@ void MainWindow::showDetailsWindow()
 
     if (lastSearchDetails.duration != "")
     {
-        headerText.append("<br><b>Duration: </b>" + lastSearchDetails.duration);
+        headerText.append(tr("<br><b>Duration: </b>") + lastSearchDetails.duration);
     }
 
     if (lastSearchDetails.info != "")
@@ -491,7 +615,7 @@ void MainWindow::showResultWindow()
 
     if (lastSearchResult.items.count() == 0)
     {
-        QMaemo5InformationBox::information(this, "<br>No results found.<br>" + lastSearchResult.errorMsg + "<br>", QMaemo5InformationBox::DefaultTimeout);
+        QMaemo5InformationBox::information(this, tr("<br>No results found.<br>") + lastSearchResult.errorMsg + "<br>", QMaemo5InformationBox::DefaultTimeout);
         searchButton->setEnabled(true);
         return;
     }
@@ -510,7 +634,7 @@ void MainWindow::showResultWindow()
 
     //Result Window
     resultWindow = new QMainWindow(this);
-    resultWindow->setWindowTitle("Results");
+    resultWindow->setWindowTitle(tr("Results"));
     #ifdef Q_WS_MAEMO_5
         resultWindow->setAttribute(Qt::WA_Maemo5StackedWindow);
     #endif
@@ -579,7 +703,7 @@ void MainWindow::showResultWindow()
         itemButton->setFixedHeight(72);
         if (item.state == "+0")
         {
-            buttonText.append("<br>OnTime");
+            buttonText.append(tr("<br>OnTime"));
             itemButton->setFixedHeight(100);
         } else if (item.state != "")
         {
@@ -679,6 +803,14 @@ void MainWindow::searchButtonClicked(bool checked)
         QMaemo5DatePickSelector *dateSel = (QMaemo5DatePickSelector *)dateSelector->pickSelector();
         QMaemo5ListPickSelector *modeSel = (QMaemo5ListPickSelector *)modeButton->pickSelector();
 
+
+        int trainrestrictions = 0;
+        if (backendParser->getTrainRestrictions().count() > 0)
+        {
+            QMaemo5ListPickSelector *trainSel = (QMaemo5ListPickSelector *)traintypesButton->pickSelector();
+            trainrestrictions = trainSel->currentIndex();
+        }
+
         if (stationDeparture->valueText() != stationArrival->valueText() && stationDeparture->valueText() != tr("please select") && stationArrival->valueText() != tr("please select"))
         {
             //Store last search data
@@ -705,11 +837,11 @@ void MainWindow::searchButtonClicked(bool checked)
             }
 
             searchButton->setEnabled(false);
-            lastSearchResult = backendParser->getJourneyData(stationDeparture->valueText(), stationArrival->valueText(), "", dateSel->currentDate(), timeSel->currentTime(), modeSel->currentIndex());
+            lastSearchResult = backendParser->getJourneyData(stationDeparture->valueText(), stationArrival->valueText(), "", dateSel->currentDate(), timeSel->currentTime(), modeSel->currentIndex(), trainrestrictions);
             showResultWindow();
         } else
         {
-            QMaemo5InformationBox::information(this, "<br>Please select a from and to station and make sure they are not the same.<br>" + lastSearchResult.errorMsg, QMaemo5InformationBox::DefaultTimeout);
+            QMaemo5InformationBox::information(this, tr("<br>Please select a from and to station and make sure they are not the same.<br>") + lastSearchResult.errorMsg, QMaemo5InformationBox::DefaultTimeout);
         }
     }
 }
