@@ -49,7 +49,7 @@
 #include "configdialog.h"
 #include "fahrplanutils.h"
 
-#define APP_VER "0.0.14"
+#define APP_VER "0.0.15"
 
 //Use the QtMobility namespace
 QTM_USE_NAMESPACE
@@ -76,6 +76,7 @@ public slots:
 private slots:
     void searchButtonClicked(bool checked = false);
     void aboutButtonClicked();
+    void donateButtonClicked();
     void alarmButtonClicked();
     void settingsButtonClicked();
     void earlierButtonClicked();
@@ -86,6 +87,8 @@ private slots:
     void gpsButtonClicked();
     void windowDestroyed(QObject* obj = 0);
     void resultItemClicked();
+    void selectTargetCalendar(QStringList items);
+    void addToCalendar(QString calendarname);
 
 private:
     QPushButton        *searchButton;
@@ -94,6 +97,7 @@ private:
     QMaemo5ValueButton *dateSelector;
     QMaemo5ValueButton *timeSelector;
     QMaemo5ValueButton *modeButton;
+    QMaemo5ValueButton *traintypesButton;
     ResultInfo         lastSearchResult;
     DetailResultInfo   lastSearchDetails;
     QPointer<QMainWindow> resultWindow;
