@@ -156,6 +156,15 @@ ResultInfo parserMobileBahnDe::parseJourneyData(QByteArray data)
     return result;
 }
 
+QStringList parserMobileBahnDe::getTrainRestrictions()
+{
+    QStringList result;
+    result.append(tr("All"));
+    result.append(tr("All without ICE"));
+    result.append(tr("Only local transport"));
+    return result;
+}
+
 DetailResultInfo parserMobileBahnDe::getJourneyDetailsData(QString queryUrl)
 {
     QUrl url(queryUrl);
@@ -347,10 +356,19 @@ ResultInfo parserMobileBahnDe::getJourneyData(QString queryUrl)
     return parseJourneyData(filebuffer->buffer());
 }
 
-ResultInfo parserMobileBahnDe::getJourneyData(QString destinationStation, QString arrivalStation, QString viaStation, QDate date, QTime time, int mode)
+ResultInfo parserMobileBahnDe::getJourneyData(QString destinationStation, QString arrivalStation, QString viaStation, QDate date, QTime time, int mode, int trainrestrictions)
 {
     Q_UNUSED(viaStation);
 
+    QString trainrestr = "1:1111111111000000";
+    if (trainrestrictions == 0) {
+        trainrestr = "1:1111111111000000"; //ALL
+    } else if (trainrestrictions == 1) {
+        trainrestr = "2:0111111111000000"; //All without ICE
+    } else if (trainrestrictions == 2) {
+        trainrestr = "4:00011111110000000"; //Only local transport
+    }
+
     QUrl url("http://mobile.bahn.de/bin/mobil/query.exe/eox?rt=1&use_realtime_filter=1&searchMode=NORMAL");
 
     //Postdata for normal Search
@@ -364,6 +382,8 @@ ResultInfo parserMobileBahnDe::getJourneyData(QString destinationStation, QStrin
                           time.toString("hh:mm") +
                           "&REQ0HafasSearchForw=" +
                           QString::number(mode) +
+                          "&REQ0JourneyProduct_prod_list=" +
+                          trainrestr +
                           "&existOptimizePrice=1&REQ0HafasOptimize1=0%3A1&existProductNahverkehr=yes&REQ0Tariff_TravellerAge.1=35&start=Suchen&REQ0Tariff_Class=2&REQ0Tariff_TravellerReductionClass.1=0";
 
     http->setHost(url.host(), QHttp::ConnectionModeHttp, url.port() == -1 ? 0 : url.port());
